-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
--Mandatory controller parameters
M.type = "main"
M.relevantDevice = nil
M.engineInfo = {}
M.fireEngineTemperature = 0
M.throttle = 0
M.brake = 0
M.clutchRatio = 0

--SPECIAL TO THE ARROW
local engineLtopNode = nil
local engineLbottomNode = nil
local engineRtopNode = nil
local engineRbottomNode = nil
local topThrustNodeL = nil
local topThrustNodeC = nil
local topThrustNodeR = nil
local bottomThrustNodeL = nil
local bottomThrustNodeC = nil
local bottomThrustNodeR = nil
local particleNode = nil
local enginePower = 0
local engineWarmup = 0
local engineDecayStartAltitude = 15240
local engineDecayEndAltitude = 27432
local halfDecayBoost = 1

local engineBreakBeam = nil
local engineBroken = false
local framesSinceReset = 0

--LOCAL--
local modes = {"Direct", "Gradual"}

--API--
M.throttle = 0
M.elevatorInput = 0
M.rudderInput = 0
M.aileronInput = 0
M.aileronAxis = false
M.elevatorAxis = false
M.rudderAxis = false

local elevatorMode = 1
local aileronMode = 1
local rudderMode = 0

M.toggleElevatorMode = function()
  elevatorMode = elevatorMode + 1
  if elevatorMode >= #modes then elevatorMode = 0 end
  
  --what is it tho?
  gui.message("New elevator mode: " .. modes[elevatorMode+1], 3, "Aircraft")
end

M.toggleAileronMode = function()
  aileronMode = aileronMode + 1
  if aileronMode >= #modes then aileronMode = 0 end
  
  --what is it tho?
  gui.message("New aileron mode: " .. modes[aileronMode+1], 3, "Aircraft")
end

M.toggleRudderMode = function()
  rudderMode = rudderMode + 1
  if rudderMode >= #modes then rudderMode = 0 end
  
  --what is it tho?
  gui.message("New rudder mode: " .. modes[rudderMode+1], 3, "Aircraft")
end
-------
local function init()
  --init electrics
  electrics.values.flaps = 0
  electrics.values.elevator = 0
  electrics.values.rudder = 0
  electrics.values.aileron = 0
  electrics.values.aircraft_throttle = 0.01
  electrics.values.altitudeFT = 0
  electrics.values.rollDeg = 0
  electrics.values.pitchDeg = 0
  electrics.values.yawDeg = 0
  electrics.values.landingGear = 0
  electrics.values.gForceMin = 0
  electrics.values.gForceMax = 0
  electrics.values.gForceCur = 0
  electrics.values.altitudeFTRadar = 0
  electrics.values.afterburner = 0
  electrics.values.airbrake = 0
  
  --find engine nodes
  for k,nd in pairs(v.data.nodes) do
    if nd.name == "fus4b" then particleNode = nd end
    if nd.name == "eng0t" then engineLtopNode = nd end
    if nd.name == "eng0b" then engineLbottomNode = nd end
    if nd.name == "eng1t" then engineRtopNode = nd end
    if nd.name == "eng1b" then engineRbottomNode = nd end
    if nd.name == "fus5lt" then topThrustNodeL = nd end
    if nd.name == "fus5t" then topThrustNodeC = nd end
    if nd.name == "fus5rt" then topThrustNodeR = nd end
    if nd.name == "fus5lb" then bottomThrustNodeL = nd end
    if nd.name == "fus5b" then bottomThrustNodeC = nd end
    if nd.name == "fus5rb" then bottomThrustNodeR = nd end
  end
  
  --get engine break beam
  for k,bm in pairs(v.data.beams) do
    if bm.name ~= nil and bm.name == "engineBreak" then
      engineBreakBeam = bm
    end
  end
  engineBroken = false
  framesSinceReset = 0
  
  --get engine power if we can
  if v.data.engine ~= nil and v.data.engine.power ~= nil then
    enginePower = v.data.engine.power
  end
end

local function updateGFX(dt)
  electrics.values.throttle = math.min(math.max(input.throttle, 0), 1)
  --electrics.values.brake = electrics.values.parkingBrake
  electrics.values.clutch = math.min(math.max(input.clutch, 0), 1)
  electrics.values.clutchRatio = 1 - electrics.values.clutch
  electrics.values.gear = "N"
  electrics.values.gearIndex = 0
  electrics.values.rpm = 0
  electrics.values.oiltemp = 0
  electrics.values.watertemp = 0
  electrics.values.fuel =  0
  electrics.values.lowfuel = false
  electrics.values.checkengine = false
  electrics.values.ignition = true
  electrics.values.engineThrottle = 0
  electrics.values.running = electrics.values.ignition
  
  electrics.values.gForceCur = (sensors.gx2 + sensors.gy2) / obj:getGravity()
  electrics.values.gForceMin = math.min(electrics.values.gForceCur, electrics.values.gForceMin)
  electrics.values.gForceMax = math.max(electrics.values.gForceCur, electrics.values.gForceMax)

  local castPos = obj:getPosition()
  --electrics.values.altitudeFTRadar = be:castRay(castPos:toPoint3F(), (castPos-vec3(0,0,610)):toPoint3F())
  
  M.engineInfo = {
    0
    , 0
    , 1000
    , 1000
    , 0
    , 0
    , 0
    , 0
    , 0
    , 0
    , obj:getVelocity():length()
    , 0
    , 0
    , "manual"
    , obj:getID()
    , 0
    , 0
  } 
  
  --update some electrics
  local dirVector = obj:getDirectionVector()
  local dirVectorUp = obj:getDirectionVectorUp()
  local position = obj:getPosition()

  electrics.values.rollDeg = (dirVectorUp.x * -dirVector.y + dirVectorUp.y * dirVector.x) * 57.2958
  electrics.values.pitchDeg = dirVector.z * 57.2958
  electrics.values.yawDeg = obj:getDirection() * 57.2958
  electrics.values.altitudeFT = position.z * 3.28084
  
  
  --update control surfaces
  if not M.elevatorAxis then
    if elevatorMode == 0 then
      electrics.values.elevator = M.elevatorInput
    else
      local newValue = electrics.values.elevator + ((M.elevatorInput / 2) * dt)
      electrics.values.elevator = math.min(math.max(newValue, -1), 1)
    end
  end
  
  if not M.rudderAxis then
    if rudderMode == 0 then
      electrics.values.rudder = M.rudderInput
    else
      local newValue = electrics.values.rudder + M.rudderInput * dt
      electrics.values.rudder = math.min(math.max(newValue, -1), 1)    
    end
  end
  
  if not M.aileronAxis then
    if aileronMode == 0 then
      electrics.values.aileron = M.aileronInput
    else
      local newValue = electrics.values.aileron + ((M.aileronInput / 2) * dt)
      electrics.values.aileron = math.min(math.max(newValue, -1), 1)    
    end
  end
  
  --actual aircraft shit
  if M.throttle ~= 0 then
    electrics.values.aircraft_throttle = electrics.values.aircraft_throttle +  ((dt / 5) * M.throttle)
    if electrics.values.aircraft_throttle > 0.99 then electrics.values.aircraft_throttle = 0.99 end  --hack for sounds
    if electrics.values.aircraft_throttle <= 0 then electrics.values.aircraft_throttle  = 0.01 end
    
  end
  
  if electrics.values.aircraft_throttle > 0.95 then electrics.values.afterburner = 1 end
  if electrics.values.aircraft_throttle < 0.95 then electrics.values.afterburner = 0 end
  
  --engine machine bork 
  if engineBreakBeam ~= nil and (obj:beamIsBroken(engineBreakBeam.cid) or obj:getBeamLengthRefRatio(engineBreakBeam.cid) < 0.975) or obj:inWater(engineBreakBeam.id1) or obj:inWater(engineBreakBeam.id2) and engineBroken == false and framesSinceReset > 4 then
      engineBroken = true
  end
  
  if engineBroken == true then
      electrics.values.aircraft_throttle = 0
      M.throttle = 0
  end
  
  framesSinceReset = framesSinceReset + 1
end

local function update()
  --ptx
  obj:addParticleByNodesRelative(particleNode.cid, bottomThrustNodeC.cid, 15 * electrics.values.aircraft_throttle, 40, 1, electrics.values.aircraft_throttle * 2)
  
  local currentAltitude = obj:getPosition().z
  if currentAltitude > engineDecayEndAltitude then return end --engines machine broke
  
  local decayAmount = 1
  if currentAltitude > engineDecayStartAltitude then
    local tDecayMax = (engineDecayEndAltitude - engineDecayStartAltitude)
    local tCurrentAltitude = (currentAltitude - engineDecayStartAltitude)
    decayAmount = tCurrentAltitude / tDecayMax
  end
  
  local thrustAmount = (electrics.values.aircraft_throttle * enginePower) * decayAmount
  
  --apply thrust
  obj:applyForce(engineLbottomNode.cid, bottomThrustNodeL.cid , thrustAmount)
  obj:applyForce(engineLbottomNode.cid, bottomThrustNodeC.cid , thrustAmount)
  obj:applyForce(engineLtopNode.cid, topThrustNodeL.cid , thrustAmount)
  obj:applyForce(engineLtopNode.cid, topThrustNodeC.cid , thrustAmount)
  
  obj:applyForce(engineRbottomNode.cid, bottomThrustNodeR.cid , thrustAmount)
  obj:applyForce(engineRbottomNode.cid, bottomThrustNodeC.cid , thrustAmount)
  obj:applyForce(engineRtopNode.cid, topThrustNodeR.cid , thrustAmount)
  obj:applyForce(engineRtopNode.cid, topThrustNodeC.cid , thrustAmount)
end

M.init = init
M.reset = function() M.init() end
M.update = update
M.updateGFX = updateGFX

--Mandatory main controller API
M.shiftUp = nop
M.shiftDown = nop
M.shiftToGearIndex = nop
M.cycleGearboxModes = nop
M.setGearboxMode = nop
M.setStarter = nop
M.setFreeze = nop
M.sendTorqueData = nop
-------------------------------

return M